@@ -13,15 +13,41 @@ static const gchar *test_empty_string = "";
 static const gchar *test_empty_array_string = "[ ]";
 static const gchar *test_empty_object_string = "{ }";
 
+static void
+verify_int_value (JsonNode *node)
+{
+  g_assert_cmpint (42, ==, json_node_get_int (node));
+}
+
+static void
+verify_boolean_value (JsonNode *node)
+{
+  g_assert_cmpint (TRUE, ==, json_node_get_boolean (node));
+}
+
+static void
+verify_string_value (JsonNode *node)
+{
+  g_assert_cmpstr ("string", ==, json_node_get_string (node));
+}
+
+static void
+verify_double_value (JsonNode *node)
+{
+  g_assert_cmpfloat (10.2e3, ==, json_node_get_double (node));
+}
+
 static const struct {
   const gchar *str;
   JsonNodeType type;
   GType gtype;
+  void (* verify_value) (JsonNode *node);
 } test_base_values[] = {
-  { "null", JSON_NODE_NULL, G_TYPE_INVALID },
-  { "42", JSON_NODE_VALUE, G_TYPE_INT64 },
-  { "true", JSON_NODE_VALUE, G_TYPE_BOOLEAN },
-  { "\"string\"", JSON_NODE_VALUE, G_TYPE_STRING }
+  { "null",       JSON_NODE_NULL,  G_TYPE_INVALID, NULL, },
+  { "42",         JSON_NODE_VALUE, G_TYPE_INT64,   verify_int_value },
+  { "true",       JSON_NODE_VALUE, G_TYPE_BOOLEAN, verify_boolean_value },
+  { "\"string\"", JSON_NODE_VALUE, G_TYPE_STRING,  verify_string_value },
+  { "10.2e3",     JSON_NODE_VALUE, G_TYPE_DOUBLE,  verify_double_value }
 };
 
 static const struct {
@@ -55,6 +81,7 @@ static const struct {
   GType gtype;
 } test_simple_objects[] = {
   { "{ \"test\" : 42 }", 1, "test", JSON_NODE_VALUE, G_TYPE_INT64 },
+  { "{ \"name\" : \"\", \"state\" : 1 }", 2, "name", JSON_NODE_VALUE, G_TYPE_STRING },
   { "{ \"foo\" : \"bar\", \"baz\" : null }", 2, "baz", JSON_NODE_NULL, G_TYPE_INVALID },
   { "{ \"channel\" : \"/meta/connect\" }", 1, "channel", JSON_NODE_VALUE, G_TYPE_STRING }
 };
@@ -153,6 +180,7 @@ test_base_value (void)
 
           root = json_parser_get_root (parser);
           g_assert (root != NULL);
+          g_assert (json_node_get_parent (root) == NULL);
 
           if (g_test_verbose ())
             g_print ("checking root node is of the desired type %s...\n",
@@ -160,6 +188,9 @@ test_base_value (void)
                                                                  : g_type_name (test_base_values[i].gtype));
           g_assert_cmpint (JSON_NODE_TYPE (root), ==, test_base_values[i].type);
           g_assert_cmpint (json_node_get_value_type (root), ==, test_base_values[i].gtype);
+
+          if (test_base_values[i].verify_value)
+            test_base_values[i].verify_value (root);
        }
     }
 
@@ -197,6 +228,7 @@ test_empty_array (void)
         g_print ("checking root node is an array...\n");
       root = json_parser_get_root (parser);
       g_assert_cmpint (JSON_NODE_TYPE (root), ==, JSON_NODE_ARRAY);
+      g_assert (json_node_get_parent (root) == NULL);
 
       array = json_node_get_array (root);
       g_assert (array != NULL);
@@ -245,6 +277,7 @@ test_simple_array (void)
             g_print ("checking root node is an array...\n");
           root = json_parser_get_root (parser);
           g_assert_cmpint (JSON_NODE_TYPE (root), ==, JSON_NODE_ARRAY);
+          g_assert (json_node_get_parent (root) == NULL);
 
           array = json_node_get_array (root);
           g_assert (array != NULL);
@@ -260,6 +293,7 @@ test_simple_array (void)
                      g_type_name (test_simple_arrays[i].gtype));
           node = json_array_get_element (array, test_simple_arrays[i].element);
           g_assert (node != NULL);
+          g_assert (json_node_get_parent (node) == root);
           g_assert_cmpint (JSON_NODE_TYPE (node), ==, test_simple_arrays[i].type);
           g_assert_cmpint (json_node_get_value_type (node), ==, test_simple_arrays[i].gtype);
         }
@@ -304,6 +338,7 @@ test_nested_array (void)
             g_print ("checking root node is an array...\n");
           root = json_parser_get_root (parser);
           g_assert_cmpint (JSON_NODE_TYPE (root), ==, JSON_NODE_ARRAY);
+          g_assert (json_node_get_parent (root) == NULL);
 
           array = json_node_get_array (root);
           g_assert (array != NULL);
@@ -347,7 +382,9 @@ test_empty_object (void)
       if (g_test_verbose ())
         g_print ("checking root node is an object...\n");
       root = json_parser_get_root (parser);
+      g_assert (json_node_get_parent (root) == NULL);
       g_assert_cmpint (JSON_NODE_TYPE (root), ==, JSON_NODE_OBJECT);
+      g_assert (json_node_get_parent (root) == NULL);
 
       object = json_node_get_object (root);
       g_assert (object != NULL);
@@ -396,6 +433,7 @@ test_simple_object (void)
             g_print ("checking root node is an object...\n");
           root = json_parser_get_root (parser);
           g_assert_cmpint (JSON_NODE_TYPE (root), ==, JSON_NODE_OBJECT);
+          g_assert (json_node_get_parent (root) == NULL);
 
           object = json_node_get_object (root);
           g_assert (object != NULL);
@@ -411,6 +449,7 @@ test_simple_object (void)
                      g_type_name (test_simple_objects[i].gtype));
           node = json_object_get_member (object, test_simple_objects[i].member);
           g_assert (node != NULL);
+          g_assert (json_node_get_parent (node) == root);
           g_assert_cmpint (JSON_NODE_TYPE (node), ==, test_simple_objects[i].type);
           g_assert_cmpint (json_node_get_value_type (node), ==, test_simple_objects[i].gtype);
         }
@@ -455,6 +494,7 @@ test_nested_object (void)
             g_print ("checking root node is an object...\n");
           root = json_parser_get_root (parser);
           g_assert_cmpint (JSON_NODE_TYPE (root), ==, JSON_NODE_OBJECT);
+          g_assert (json_node_get_parent (root) == NULL);
 
           object = json_node_get_object (root);
           g_assert (object != NULL);
